@@ -7,7 +7,7 @@ if [ "$TRAVIS_OS_NAME" == "osx" ]; then
     source .venv/bin/activate
 fi
 
-tox
+tox --skip-missing-interpreters false
 
 # re-run all the XML-related tests, this time without lxml but using the
 # built-in ElementTree library.
@@ -17,4 +17,4 @@ else
     # strip additional tox envs after the comma, add -nolxml factor
     TOXENV="${TOXENV%,*}-nolxml"
 fi
-tox -e $TOXENV -- Tests/ufoLib Tests/misc/etree_test.py Tests/misc/plistlib_test.py
+tox --skip-missing-interpreters false -e $TOXENV -- Tests/ufoLib Tests/misc/etree_test.py Tests/misc/plistlib_test.py
