@@ -22,6 +22,9 @@ def reload_font(font):
     """(De)serialize to get final binary layout."""
     buf = BytesIO()
     font.save(buf)
+    # Close the font to release filesystem resources so that on Windows the tearDown
+    # method can successfully remove the temporary directory created during setUp.
+    font.close()
     buf.seek(0)
     return TTFont(buf)
 
@@ -218,6 +221,30 @@ class BuildTest(unittest.TestCase):
             save_before_dump=True,
         )
 
+    def test_varlib_build_feature_variations_whole_range(self):
+        """Designspace file contains <rules> element specifying the entire design
+        space, used to build GSUB FeatureVariations table.
+        """
+        self._run_varlib_build_test(
+            designspace_name="FeatureVarsWholeRange",
+            font_name="TestFamily",
+            tables=["fvar", "GSUB"],
+            expected_ttx_name="FeatureVarsWholeRange",
+            save_before_dump=True,
+        )
+
+    def test_varlib_build_feature_variations_whole_range_empty(self):
+        """Designspace file contains <rules> element without a condition, specifying
+        the entire design space, used to build GSUB FeatureVariations table.
+        """
+        self._run_varlib_build_test(
+            designspace_name="FeatureVarsWholeRangeEmpty",
+            font_name="TestFamily",
+            tables=["fvar", "GSUB"],
+            expected_ttx_name="FeatureVarsWholeRange",
+            save_before_dump=True,
+        )
+
     def test_varlib_build_feature_variations_with_existing_rclt(self):
         """Designspace file contains <rules> element, used to build GSUB
         FeatureVariations table. <rules> is specified to do its OT processing
@@ -628,6 +655,31 @@ class BuildTest(unittest.TestCase):
         self.expect_ttx(varfont, expected_ttx_path, tables)
         self.check_ttx_dump(varfont, expected_ttx_path, tables, suffix)
 
+    def test_varlib_build_BASE(self):
+        ds_path = self.get_test_input('TestBASE.designspace')
+        ttx_dir = self.get_test_input("master_base_test")
+        expected_ttx_name = 'TestBASE'
+        suffix = '.otf'
+
+        self.temp_dir()
+        for path in self.get_file_list(ttx_dir, '.ttx', 'TestBASE'):
+            font, savepath = self.compile_font(path, suffix, self.tempdir)
+
+        ds = DesignSpaceDocument.fromfile(ds_path)
+        for source in ds.sources:
+            source.path = os.path.join(
+                self.tempdir, os.path.basename(source.filename).replace(".ufo", suffix)
+            )
+        ds.updatePaths()
+
+        varfont, _, _ = build(ds)
+        varfont = reload_font(varfont)
+
+        expected_ttx_path = self.get_test_output(expected_ttx_name + '.ttx')
+        tables = ["BASE"]
+        self.expect_ttx(varfont, expected_ttx_path, tables)
+        self.check_ttx_dump(varfont, expected_ttx_path, tables, suffix)
+
     def test_varlib_build_single_master(self):
         self._run_varlib_build_test(
             designspace_name='SingleMaster',
